@@ -488,9 +488,9 @@ void write_params(const stan::mcmc::chains<> &chains,
         *out << std::right;
         for (int j = 0; j < params.cols(); j++) {
           std::cout.setf(col_formats(j), std::ios::floatfield);
-          *out << std::setprecision(compute_precision(
-                      params(i, j), sig_figs,
-                      col_formats(j) == std::ios_base::scientific))
+          *out << std::setprecision(
+              compute_precision(params(i, j), sig_figs,
+                                col_formats(j) == std::ios_base::scientific))
                << std::setw(col_widths(j)) << params(i, j);
         }
       }
@@ -520,9 +520,9 @@ void write_params(const stan::mcmc::chains<> &chains,
           *out << std::right;
           for (int j = 0; j < params.cols(); j++) {
             std::cout.setf(col_formats(j), std::ios::floatfield);
-            *out << std::setprecision(compute_precision(
-                        params(row_maj_index, j), sig_figs,
-                        col_formats(j) == std::ios_base::scientific))
+            *out << std::setprecision(
+                compute_precision(params(row_maj_index, j), sig_figs,
+                                  col_formats(j) == std::ios_base::scientific))
                  << std::setw(col_widths(j)) << params(row_maj_index, j);
           }
         }
