@@ -22,16 +22,27 @@
 #include <stdlib.h>
 
 #include <sundials/sundials_nvector.h>
+#include "sundials_context_impl.h"
+
+#if defined(SUNDIALS_BUILD_WITH_PROFILING)
+static SUNProfiler getSUNProfiler(N_Vector v)
+{
+  return(v->sunctx->profiler);
+}
+#endif
 
 /* -----------------------------------------------------------------
- * Create an empty NVector object
+ * Methods that are not ops (i.e., non-virtual and not overridable).
  * -----------------------------------------------------------------*/
 
-N_Vector N_VNewEmpty()
+/* Create an empty NVector object */
+N_Vector N_VNewEmpty(SUNContext sunctx)
 {
   N_Vector     v;
   N_Vector_Ops ops;
 
+  if (sunctx == NULL) return(NULL);
+
   /* create vector object */
   v = NULL;
   v = (N_Vector) malloc(sizeof *v);
@@ -44,6 +55,12 @@ N_Vector N_VNewEmpty()
 
   /* initialize operations to NULL */
 
+  /*
+   * REQUIRED operations.
+   *
+   * These must be implemented by derivations of the generic N_Vector.
+   */
+
   /* constructors, destructors, and utility operations */
   ops->nvgetvectorid           = NULL;
   ops->nvclone                 = NULL;
@@ -67,8 +84,8 @@ N_Vector N_VNewEmpty()
   ops->nvaddconst     = NULL;
   ops->nvdotprod      = NULL;
   ops->nvmaxnorm      = NULL;
-  ops->nvwrmsnormmask = NULL;
   ops->nvwrmsnorm     = NULL;
+  ops->nvwrmsnormmask = NULL;
   ops->nvmin          = NULL;
   ops->nvwl2norm      = NULL;
   ops->nvl1norm       = NULL;
@@ -77,6 +94,12 @@ N_Vector N_VNewEmpty()
   ops->nvconstrmask   = NULL;
   ops->nvminquotient  = NULL;
 
+  /*
+   * OPTIONAL operations.
+   *
+   * These operations provide default implementations that may be overriden.
+   */
+
   /* fused vector operations (optional) */
   ops->nvlinearcombination = NULL;
   ops->nvscaleaddmulti     = NULL;
@@ -91,16 +114,24 @@ N_Vector N_VNewEmpty()
   ops->nvscaleaddmultivectorarray     = NULL;
   ops->nvlinearcombinationvectorarray = NULL;
 
+  /*
+   * OPTIONAL operations with no default implementation.
+   */
+
   /* local reduction operations (optional) */
-  ops->nvdotprodlocal     = NULL;
-  ops->nvmaxnormlocal     = NULL;
-  ops->nvminlocal         = NULL;
-  ops->nvl1normlocal      = NULL;
-  ops->nvinvtestlocal     = NULL;
-  ops->nvconstrmasklocal  = NULL;
-  ops->nvminquotientlocal = NULL;
-  ops->nvwsqrsumlocal     = NULL;
-  ops->nvwsqrsummasklocal = NULL;
+  ops->nvdotprodlocal      = NULL;
+  ops->nvmaxnormlocal      = NULL;
+  ops->nvminlocal          = NULL;
+  ops->nvl1normlocal       = NULL;
+  ops->nvinvtestlocal      = NULL;
+  ops->nvconstrmasklocal   = NULL;
+  ops->nvminquotientlocal  = NULL;
+  ops->nvwsqrsumlocal      = NULL;
+  ops->nvwsqrsummasklocal  = NULL;
+
+  /* single buffer reduction operations */
+  ops->nvdotprodmultilocal     = NULL;
+  ops->nvdotprodmultiallreduce = NULL;
 
   /* XBraid interface operations */
   ops->nvbufsize   = NULL;
@@ -111,18 +142,17 @@ N_Vector N_VNewEmpty()
   ops->nvprint     = NULL;
   ops->nvprintfile = NULL;
 
-  /* attach ops and initialize content to NULL */
-  v->ops     = ops;
+  /* attach ops */
+  v->ops = ops;
+
+  /* initialize content and sunctx to NULL */
   v->content = NULL;
+  v->sunctx  = sunctx;
 
   return(v);
 }
 
-
-/* -----------------------------------------------------------------
- * Free a generic N_Vector (assumes content is already empty)
- * -----------------------------------------------------------------*/
-
+/* Free a generic N_Vector (assumes content is already empty) */
 void N_VFreeEmpty(N_Vector v)
 {
   if (v == NULL)  return;
@@ -136,11 +166,7 @@ void N_VFreeEmpty(N_Vector v)
   return;
 }
 
-
-/* -----------------------------------------------------------------
- * Copy a vector 'ops' structure
- * -----------------------------------------------------------------*/
-
+/* Copy a vector 'ops' structure */
 int N_VCopyOps(N_Vector w, N_Vector v)
 {
   /* Check that ops structures exist */
@@ -149,6 +175,12 @@ int N_VCopyOps(N_Vector w, N_Vector v)
 
   /* Copy ops from w to v */
 
+  /*
+   * REQUIRED operations.
+   *
+   * These must be implemented by derivations of the generic N_Vector.
+   */
+
   /* constructors, destructors, and utility operations */
   v->ops->nvgetvectorid           = w->ops->nvgetvectorid;
   v->ops->nvclone                 = w->ops->nvclone;
@@ -172,8 +204,8 @@ int N_VCopyOps(N_Vector w, N_Vector v)
   v->ops->nvaddconst     = w->ops->nvaddconst;
   v->ops->nvdotprod      = w->ops->nvdotprod;
   v->ops->nvmaxnorm      = w->ops->nvmaxnorm;
-  v->ops->nvwrmsnormmask = w->ops->nvwrmsnormmask;
   v->ops->nvwrmsnorm     = w->ops->nvwrmsnorm;
+  v->ops->nvwrmsnormmask = w->ops->nvwrmsnormmask;
   v->ops->nvmin          = w->ops->nvmin;
   v->ops->nvwl2norm      = w->ops->nvwl2norm;
   v->ops->nvl1norm       = w->ops->nvl1norm;
@@ -182,6 +214,12 @@ int N_VCopyOps(N_Vector w, N_Vector v)
   v->ops->nvconstrmask   = w->ops->nvconstrmask;
   v->ops->nvminquotient  = w->ops->nvminquotient;
 
+  /*
+   * OPTIONAL operations.
+   *
+   * These operations provide default implementations that may be overriden.
+   */
+
   /* fused vector operations */
   v->ops->nvlinearcombination = w->ops->nvlinearcombination;
   v->ops->nvscaleaddmulti     = w->ops->nvscaleaddmulti;
@@ -196,16 +234,24 @@ int N_VCopyOps(N_Vector w, N_Vector v)
   v->ops->nvscaleaddmultivectorarray     = w->ops->nvscaleaddmultivectorarray;
   v->ops->nvlinearcombinationvectorarray = w->ops->nvlinearcombinationvectorarray;
 
+  /*
+   * OPTIONAL operations with no default implementation.
+   */
+
   /* local reduction operations */
-  v->ops->nvdotprodlocal     = w->ops->nvdotprodlocal;
-  v->ops->nvmaxnormlocal     = w->ops->nvmaxnormlocal;
-  v->ops->nvminlocal         = w->ops->nvminlocal;
-  v->ops->nvl1normlocal      = w->ops->nvl1normlocal;
-  v->ops->nvinvtestlocal     = w->ops->nvinvtestlocal;
-  v->ops->nvconstrmasklocal  = w->ops->nvconstrmasklocal;
-  v->ops->nvminquotientlocal = w->ops->nvminquotientlocal;
-  v->ops->nvwsqrsumlocal     = w->ops->nvwsqrsumlocal;
-  v->ops->nvwsqrsummasklocal = w->ops->nvwsqrsummasklocal;
+  v->ops->nvdotprodlocal      = w->ops->nvdotprodlocal;
+  v->ops->nvmaxnormlocal      = w->ops->nvmaxnormlocal;
+  v->ops->nvminlocal          = w->ops->nvminlocal;
+  v->ops->nvl1normlocal       = w->ops->nvl1normlocal;
+  v->ops->nvinvtestlocal      = w->ops->nvinvtestlocal;
+  v->ops->nvconstrmasklocal   = w->ops->nvconstrmasklocal;
+  v->ops->nvminquotientlocal  = w->ops->nvminquotientlocal;
+  v->ops->nvwsqrsumlocal      = w->ops->nvwsqrsumlocal;
+  v->ops->nvwsqrsummasklocal  = w->ops->nvwsqrsummasklocal;
+
+  /* single buffer reduction operations */
+  v->ops->nvdotprodmultilocal     = w->ops->nvdotprodmultilocal;
+  v->ops->nvdotprodmultiallreduce = w->ops->nvdotprodmultiallreduce;
 
   /* XBraid interface operations */
   v->ops->nvbufsize   = w->ops->nvbufsize;
@@ -230,12 +276,22 @@ N_Vector_ID N_VGetVectorID(N_Vector w)
 
 N_Vector N_VClone(N_Vector w)
 {
-  return(w->ops->nvclone(w));
+  N_Vector result = NULL;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(w));
+  result = w->ops->nvclone(w);
+  result->sunctx = w->sunctx;
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(w));
+  return result;
 }
 
 N_Vector N_VCloneEmpty(N_Vector w)
 {
-  return(w->ops->nvcloneempty(w));
+  N_Vector result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(w));
+  result = w->ops->nvcloneempty(w);
+  result->sunctx = w->sunctx;
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(w));
+  return result;
 }
 
 void N_VDestroy(N_Vector v)
@@ -299,167 +355,237 @@ sunindextype N_VGetLength(N_Vector v)
 
 void N_VLinearSum(realtype a, N_Vector x, realtype b, N_Vector y, N_Vector z)
 {
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
   z->ops->nvlinearsum(a, x, b, y, z);
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
   return;
 }
 
 void N_VConst(realtype c, N_Vector z)
 {
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(z));
   z->ops->nvconst(c, z);
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(z));
   return;
 }
 
 void N_VProd(N_Vector x, N_Vector y, N_Vector z)
 {
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
   z->ops->nvprod(x, y, z);
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
   return;
 }
 
 void N_VDiv(N_Vector x, N_Vector y, N_Vector z)
 {
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
   z->ops->nvdiv(x, y, z);
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
   return;
 }
 
 void N_VScale(realtype c, N_Vector x, N_Vector z)
 {
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
   z->ops->nvscale(c, x, z);
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
   return;
 }
 
 void N_VAbs(N_Vector x, N_Vector z)
 {
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
   z->ops->nvabs(x, z);
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
   return;
 }
 
 void N_VInv(N_Vector x, N_Vector z)
 {
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
   z->ops->nvinv(x, z);
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
   return;
 }
 
 void N_VAddConst(N_Vector x, realtype b, N_Vector z)
 {
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
   z->ops->nvaddconst(x, b, z);
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
   return;
 }
 
 realtype N_VDotProd(N_Vector x, N_Vector y)
 {
-  return((realtype) y->ops->nvdotprod(x, y));
+  realtype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  result = ((realtype) y->ops->nvdotprod(x, y));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(result);
 }
 
 realtype N_VMaxNorm(N_Vector x)
 {
-  return((realtype) x->ops->nvmaxnorm(x));
+  realtype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  result = ((realtype) x->ops->nvmaxnorm(x));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(result);
 }
 
 realtype N_VWrmsNorm(N_Vector x, N_Vector w)
 {
-  return((realtype) x->ops->nvwrmsnorm(x, w));
+  realtype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  result = ((realtype) x->ops->nvwrmsnorm(x, w));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(result);
 }
 
 realtype N_VWrmsNormMask(N_Vector x, N_Vector w, N_Vector id)
 {
-  return((realtype) x->ops->nvwrmsnormmask(x, w, id));
+  realtype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  result = ((realtype) x->ops->nvwrmsnormmask(x, w, id));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(result);
 }
 
 realtype N_VMin(N_Vector x)
 {
-  return((realtype) x->ops->nvmin(x));
+  realtype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  result = ((realtype) x->ops->nvmin(x));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(result);
 }
 
 realtype N_VWL2Norm(N_Vector x, N_Vector w)
 {
-  return((realtype) x->ops->nvwl2norm(x, w));
+  realtype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  result = ((realtype) x->ops->nvwl2norm(x, w));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(result);
 }
 
 realtype N_VL1Norm(N_Vector x)
 {
-  return((realtype) x->ops->nvl1norm(x));
+  realtype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  result = ((realtype) x->ops->nvl1norm(x));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(result);
 }
 
 void N_VCompare(realtype c, N_Vector x, N_Vector z)
 {
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
   z->ops->nvcompare(c, x, z);
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
   return;
 }
 
 booleantype N_VInvTest(N_Vector x, N_Vector z)
 {
-  return((booleantype) z->ops->nvinvtest(x, z));
+  booleantype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  result = ((booleantype) z->ops->nvinvtest(x, z));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(result);
 }
 
 booleantype N_VConstrMask(N_Vector c, N_Vector x, N_Vector m)
 {
-  return((booleantype) x->ops->nvconstrmask(c, x, m));
+  booleantype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(c));
+  result = ((booleantype) x->ops->nvconstrmask(c, x, m));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(c));
+  return(result);
 }
 
 realtype N_VMinQuotient(N_Vector num, N_Vector denom)
 {
-  return((realtype) num->ops->nvminquotient(num, denom));
+  realtype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(num));
+  result = ((realtype) num->ops->nvminquotient(num, denom));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(num));
+  return(result);
 }
 
+
+
 /* -----------------------------------------------------------------
  * OPTIONAL fused vector operations
  * -----------------------------------------------------------------*/
 
 int N_VLinearCombination(int nvec, realtype* c, N_Vector* X, N_Vector z)
 {
-  int i;
-  realtype ONE=RCONST(1.0);
+  int i, ier;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(X[0]));
 
   if (z->ops->nvlinearcombination != NULL) {
 
-    return(z->ops->nvlinearcombination(nvec, c, X, z));
+    ier = z->ops->nvlinearcombination(nvec, c, X, z);
 
   } else {
 
     z->ops->nvscale(c[0], X[0], z);
     for (i=1; i<nvec; i++) {
-      z->ops->nvlinearsum(c[i], X[i], ONE, z, z);
+      z->ops->nvlinearsum(c[i], X[i], RCONST(1.0), z, z);
     }
-    return(0);
+    ier = 0;
 
   }
+
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(X[0]));
+  return(ier);
 }
 
 int N_VScaleAddMulti(int nvec, realtype* a, N_Vector x, N_Vector* Y, N_Vector* Z)
 {
-  int i;
-  realtype ONE=RCONST(1.0);
+  int i, ier;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
 
   if (x->ops->nvscaleaddmulti != NULL) {
 
-    return(x->ops->nvscaleaddmulti(nvec, a, x, Y, Z));
+    ier = x->ops->nvscaleaddmulti(nvec, a, x, Y, Z);
 
   } else {
 
     for (i=0; i<nvec; i++) {
-      x->ops->nvlinearsum(a[i], x, ONE, Y[i], Z[i]);
+      x->ops->nvlinearsum(a[i], x, RCONST(1.0), Y[i], Z[i]);
     }
-    return(0);
+    ier = 0;
 
   }
+
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(ier);
 }
 
 int N_VDotProdMulti(int nvec, N_Vector x, N_Vector* Y, realtype* dotprods)
 {
-  int i;
+  int i, ier;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
 
   if (x->ops->nvdotprodmulti != NULL) {
 
-    return(x->ops->nvdotprodmulti(nvec, x, Y, dotprods));
+    ier = x->ops->nvdotprodmulti(nvec, x, Y, dotprods);
 
   } else {
 
     for (i=0; i<nvec; i++) {
       dotprods[i] = x->ops->nvdotprod(x, Y[i]);
     }
-    return(0);
+    ier = 0;
 
   }
+
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(ier);
 }
 
 /* -----------------------------------------------------------------
@@ -469,111 +595,126 @@ int N_VDotProdMulti(int nvec, N_Vector x, N_Vector* Y, realtype* dotprods)
 int N_VLinearSumVectorArray(int nvec, realtype a, N_Vector* X,
                             realtype b, N_Vector* Y, N_Vector* Z)
 {
-  int i;
+  int i, ier;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(X[0]));
 
   if (Z[0]->ops->nvlinearsumvectorarray != NULL) {
 
-    return(Z[0]->ops->nvlinearsumvectorarray(nvec, a, X, b, Y, Z));
+    ier = Z[0]->ops->nvlinearsumvectorarray(nvec, a, X, b, Y, Z);
 
   } else {
 
     for (i=0; i<nvec; i++) {
       Z[0]->ops->nvlinearsum(a, X[i], b, Y[i], Z[i]);
     }
-    return(0);
+    ier = 0;
 
   }
+
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(X[0]));
+  return(ier);
 }
 
 int N_VScaleVectorArray(int nvec, realtype* c, N_Vector* X, N_Vector* Z)
 {
-  int i;
+  int i, ier;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(X[0]));
 
   if (Z[0]->ops->nvscalevectorarray != NULL) {
 
-    return(Z[0]->ops->nvscalevectorarray(nvec, c, X, Z));
+    ier = Z[0]->ops->nvscalevectorarray(nvec, c, X, Z);
 
   } else {
 
     for (i=0; i<nvec; i++) {
       Z[0]->ops->nvscale(c[i], X[i], Z[i]);
     }
-    return(0);
+    ier = 0;
 
   }
+
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(X[0]));
+  return(ier);
 }
 
 int N_VConstVectorArray(int nvec, realtype c, N_Vector* Z)
 {
   int i, ier;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(Z[0]));
 
   if (Z[0]->ops->nvconstvectorarray != NULL) {
 
     ier = Z[0]->ops->nvconstvectorarray(nvec, c, Z);
-    return(ier);
 
   } else {
 
     for (i=0; i<nvec; i++) {
       Z[0]->ops->nvconst(c, Z[i]);
     }
-    return(0);
+    ier = 0;
 
   }
+
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(Z[0]));
+  return(ier);
 }
 
 int N_VWrmsNormVectorArray(int nvec, N_Vector* X, N_Vector* W, realtype* nrm)
 {
   int i, ier;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(X[0]));
 
   if (X[0]->ops->nvwrmsnormvectorarray != NULL) {
 
     ier = X[0]->ops->nvwrmsnormvectorarray(nvec, X, W, nrm);
-    return(ier);
 
   } else {
 
     for (i=0; i<nvec; i++) {
       nrm[i] = X[0]->ops->nvwrmsnorm(X[i], W[i]);
     }
-    return(0);
+    ier = 0;
 
   }
+
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(X[0]));
+  return(ier);
 }
 
 int N_VWrmsNormMaskVectorArray(int nvec, N_Vector* X, N_Vector* W, N_Vector id,
                                realtype* nrm)
 {
   int i, ier;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(X[0]));
 
   if (id->ops->nvwrmsnormmaskvectorarray != NULL) {
 
     ier = id->ops->nvwrmsnormmaskvectorarray(nvec, X, W, id, nrm);
-    return(ier);
 
   } else {
 
     for (i=0; i<nvec; i++) {
       nrm[i] = id->ops->nvwrmsnormmask(X[i], W[i], id);
     }
-    return(0);
+    ier = 0;
 
   }
+
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(X[0]));
+  return(ier);
 }
 
 int N_VScaleAddMultiVectorArray(int nvec, int nsum, realtype* a, N_Vector* X,
                                  N_Vector** Y, N_Vector** Z)
 {
-  int        i, j;
-  int        ier=0;
-  realtype   ONE=RCONST(1.0);
-  N_Vector* YY=NULL;
-  N_Vector* ZZ=NULL;
+  int i, j, ier;
+  N_Vector* YY = NULL;
+  N_Vector* ZZ = NULL;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(X[0]));
 
   if (X[0]->ops->nvscaleaddmultivectorarray != NULL) {
 
     ier = X[0]->ops->nvscaleaddmultivectorarray(nvec, nsum, a, X, Y, Z);
-    return(ier);
 
   } else if (X[0]->ops->nvscaleaddmulti != NULL ) {
 
@@ -596,31 +737,30 @@ int N_VScaleAddMultiVectorArray(int nvec, int nsum, realtype* a, N_Vector* X,
     free(YY);
     free(ZZ);
 
-    return(ier);
-
   } else {
 
     for (i=0; i<nvec; i++) {
       for (j=0; j<nsum; j++) {
-        X[0]->ops->nvlinearsum(a[j], X[i], ONE, Y[j][i], Z[j][i]);
+        X[0]->ops->nvlinearsum(a[j], X[i], RCONST(1.0), Y[j][i], Z[j][i]);
       }
     }
-    return(0);
+    ier = 0;
   }
+
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(X[0]));
+  return(ier);
 }
 
 int N_VLinearCombinationVectorArray(int nvec, int nsum, realtype* c,
                                     N_Vector** X, N_Vector* Z)
 {
-  int        i, j;
-  int        ier=0;
-  realtype   ONE=RCONST(1.0);
-  N_Vector* Y=NULL;
+  int i, j, ier;
+  N_Vector* Y = NULL;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(X[0][0]));
 
   if (Z[0]->ops->nvlinearcombinationvectorarray != NULL) {
 
     ier = Z[0]->ops->nvlinearcombinationvectorarray(nvec, nsum, c, X, Z);
-    return(ier);
 
   } else if (Z[0]->ops->nvlinearcombination != NULL ) {
 
@@ -640,18 +780,19 @@ int N_VLinearCombinationVectorArray(int nvec, int nsum, realtype* c,
     /* free array of vectors */
     free(Y);
 
-    return(ier);
-
   } else {
 
     for (i=0; i<nvec; i++) {
       Z[0]->ops->nvscale(c[0], X[0][i], Z[i]);
       for (j=1; j<nsum; j++) {
-        Z[0]->ops->nvlinearsum(c[j], X[j][i], ONE, Z[i], Z[i]);
+        Z[0]->ops->nvlinearsum(c[j], X[j][i], RCONST(1.0), Z[i], Z[i]);
       }
     }
-    return(0);
+    ier = 0;
   }
+
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(X[0][0]));
+  return(ier);
 }
 
 /* -----------------------------------------------------------------
@@ -660,47 +801,115 @@ int N_VLinearCombinationVectorArray(int nvec, int nsum, realtype* c,
 
 realtype N_VDotProdLocal(N_Vector x, N_Vector y)
 {
-  return((realtype) y->ops->nvdotprodlocal(x, y));
+  realtype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  result = ((realtype) y->ops->nvdotprodlocal(x, y));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(result);
 }
 
 realtype N_VMaxNormLocal(N_Vector x)
 {
-  return((realtype) x->ops->nvmaxnormlocal(x));
+  realtype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  result = ((realtype) x->ops->nvmaxnormlocal(x));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(result);
 }
 
 realtype N_VMinLocal(N_Vector x)
 {
-  return((realtype) x->ops->nvminlocal(x));
+  realtype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  result = ((realtype) x->ops->nvminlocal(x));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(result);
 }
 
 realtype N_VL1NormLocal(N_Vector x)
 {
-  return((realtype) x->ops->nvl1normlocal(x));
+  realtype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  result = ((realtype) x->ops->nvl1normlocal(x));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(result);
 }
 
 realtype N_VWSqrSumLocal(N_Vector x, N_Vector w)
 {
-  return((realtype) x->ops->nvwsqrsumlocal(x,w));
+  realtype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  result = ((realtype) x->ops->nvwsqrsumlocal(x,w));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(result);
 }
 
 realtype N_VWSqrSumMaskLocal(N_Vector x, N_Vector w, N_Vector id)
 {
-  return((realtype) x->ops->nvwsqrsummasklocal(x,w,id));
+  realtype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  result = ((realtype) x->ops->nvwsqrsummasklocal(x,w,id));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(result);
 }
 
 booleantype N_VInvTestLocal(N_Vector x, N_Vector z)
 {
-  return((booleantype) z->ops->nvinvtestlocal(x,z));
+  booleantype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  result = ((booleantype) z->ops->nvinvtestlocal(x,z));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(result);
 }
 
 booleantype N_VConstrMaskLocal(N_Vector c, N_Vector x, N_Vector m)
 {
-  return((booleantype) x->ops->nvconstrmasklocal(c,x,m));
+  booleantype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  result = ((booleantype) x->ops->nvconstrmasklocal(c,x,m));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(result);
 }
 
 realtype N_VMinQuotientLocal(N_Vector num, N_Vector denom)
 {
-  return((realtype) num->ops->nvminquotientlocal(num,denom));
+  realtype result;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(num));
+  result = ((realtype) num->ops->nvminquotientlocal(num,denom));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(num));
+  return(result);
+}
+
+/* -------------------------------------------
+ * OPTIONAL single buffer reduction operations
+ * -------------------------------------------*/
+
+int N_VDotProdMultiLocal(int nvec, N_Vector x, N_Vector* Y, realtype* dotprods)
+{
+  int i;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+
+  if (x->ops->nvdotprodmultilocal)
+    return((int) x->ops->nvdotprodmultilocal(nvec, x, Y, dotprods));
+
+  if (x->ops->nvdotprodlocal) {
+    for (i = 0; i < nvec; i++) {
+      dotprods[i] = x->ops->nvdotprodlocal(x, Y[i]);
+    }
+    return(0);
+  }
+
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(-1);
+}
+
+int N_VDotProdMultiAllReduce(int nvec, N_Vector x, realtype* sum)
+{
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  if (x->ops->nvdotprodmultiallreduce)
+    return(x->ops->nvdotprodmultiallreduce(nvec, x, sum));
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(-1);
 }
 
 /* ------------------------------------
@@ -709,20 +918,38 @@ realtype N_VMinQuotientLocal(N_Vector num, N_Vector denom)
 
 int N_VBufSize(N_Vector x, sunindextype *size)
 {
-  if (x->ops->nvbufsize == NULL) return(-1);
-  return(x->ops->nvbufsize(x, size));
+  int ier;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  if (x->ops->nvbufsize == NULL)
+    ier = -1;
+  else
+    ier = x->ops->nvbufsize(x, size);
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(ier);
 }
 
 int N_VBufPack(N_Vector x, void *buf)
 {
-  if (x->ops->nvbufpack == NULL) return(-1);
-  return(x->ops->nvbufpack(x, buf));
+  int ier;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  if (x->ops->nvbufpack == NULL)
+    ier = -1;
+  else
+    ier = x->ops->nvbufpack(x, buf);
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(ier);
 }
 
 int N_VBufUnpack(N_Vector x, void *buf)
 {
-  if (x->ops->nvbufunpack == NULL) return(-1);
-  return(x->ops->nvbufunpack(x, buf));
+  int ier;
+  SUNDIALS_MARK_FUNCTION_BEGIN(getSUNProfiler(x));
+  if (x->ops->nvbufunpack == NULL)
+    ier = -1;
+  else
+    ier = x->ops->nvbufunpack(x, buf);
+  SUNDIALS_MARK_FUNCTION_END(getSUNProfiler(x));
+  return(ier);
 }
 
 /* -----------------------------------------------------------------
@@ -773,7 +1000,7 @@ N_Vector* N_VCloneVectorArray(int count, N_Vector w)
   if (count <= 0) return(NULL);
 
   vs = (N_Vector* ) malloc(count * sizeof(N_Vector));
-  if(vs == NULL) return(NULL);
+  if (vs == NULL) return(NULL);
 
   for (j = 0; j < count; j++) {
     vs[j] = N_VClone(w);
@@ -790,9 +1017,12 @@ void N_VDestroyVectorArray(N_Vector* vs, int count)
 {
   int j;
 
-  if (vs==NULL) return;
+  if (vs == NULL) return;
 
-  for (j = 0; j < count; j++) N_VDestroy(vs[j]);
+  for (j = 0; j < count; j++) {
+    N_VDestroy(vs[j]);
+    vs[j] = NULL;
+  }
 
   free(vs); vs = NULL;
 
