@@ -5,8 +5,10 @@ TEST(mathMixMatFun, tanh) {
     using stan::math::tanh;
     return tanh(x1);
   };
-  stan::test::expect_common_nonzero_unary_vectorized(f);
-  stan::test::expect_unary_vectorized(f, -2.6, -2, -1.2, -0.5, 0.5, 1.5);
+  stan::test::expect_common_nonzero_unary_vectorized<
+      stan::test::PromoteToComplex::No>(f);
+  stan::test::expect_unary_vectorized<stan::test::PromoteToComplex::Yes>(
+      f, -2.6, -2, -1.2, -0.5, 0.5, 1.5);
   stan::test::expect_complex_common(f);
 }
 
