@@ -5,8 +5,8 @@
 #include <limits>
 
 void test_logical_or(double x, double y) {
-  AVAR x_v = x;
-  AVAR y_v = y;
+  stan::math::var x_v = x;
+  stan::math::var y_v = y;
   EXPECT_EQ(x || y, x_v || y_v);
 }
 
